<?php
/**
 * PercorreCAGED.
 *
 * @package Logic
 *
 * @author  thalesvalias - 22/05/2012
 */
class PercorreCAGED{

    private $_parametros;
    private $_formulario;

    private $_retrocesso;

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;

    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_tpInscHierarq;
    private $_idGeraMeioMagnetico;
    private $_idProcessaVinculo;
    private $_filePath;
    private $_nrEstruturaResp;
    private $_tpInscResp;
    private $_nrGrpIncide;
    private $_idAtualizacao;
    private $_nrTipoModalidadeCalculo;
    private $_ambienteSalarial;
    private $_tpModalidadeSalarioOperador;

    private $_calculoFolha;
    private $_listaNrCalculoFolha;
    private $_cagedMagnetico;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_vinculom;
    private $_itemGrupo;

    private $_pessoa;
    private $_estruturacao;

    private $_nomeArquivo;
    private $_caged;
    private $_registroA;

    private $_msgSucessoPersisteMagnetico;
    private $_msgSucessoGeraMagnetico;

    const ALTERACAO = '2';
    const PREFIXO_NOME_ARQUIVO_NORMAL = 'CGED';
    const PREFIXO_NOME_ARQUIVO_ACERTO = 'A01';
    const PREFIXO_EXTENSAO_ARQUIVO = '.M';


    /**
     * Construtor.
     *
     * @param string     $tpProcessamento Tipo de processamento (MENSAL / COMPETENCIA 13).
     * @param integer    $nrFormulario    Numero do formulario.
     * @param Parametros $parametro       Instancia de Parametros.
     *
     * @author thalesvalias - 22/05/2012
     */
    public function __construct($nrFormulario, Parametros $parametro, $cdOperador){

        $this->_parametros = $parametro;
        $this->_nrOrg = \Ambiente::getNrorgTrabalho();
        $this->_nrOrgPadrao = $this->_parametros->getNrOrgPadrao();
        $this->_nrEstruturaTrab = $this->_parametros->getNrEstrutTrab($this->_nrOrg, $cdOperador);
        $this->_mesCompetencia = new TkDateTime($this->_parametros->getMesCompetencia());
        $this->_nrTipoModalidadeCalculo = $this->_parametros->getTipoModalidadeCalculo();
        $this->_ambienteSalarial = $this->_parametros->ambienteSalarioOperador();
        $this->_tpModalidadeSalarioOperador = $this->_parametros->getTipoModalidadeSalario();
        $this->_tipoMovimento = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->_itemGrupo = new FpaGrpitemEntity();
        $this->_vinculom = new GpevinculomEntity();
        $this->_tipoHierarquia = new TipohierarqorgEntity();
        $this->_hierarquiaOrg = new HierarquiaorgEntity();
        $this->_calculoFolha = new FpacalculofolhaEntity();
        $this->_cagedMagnetico = new FpacagedmagnetEntity();
        $this->_registroA = new RegistroACAGED();
        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);
        $this->_inicializaVariaveis($nrFormulario);
        $this->_caged  = new CAGED($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_cagedMagnetico,
        $this->_formulario, $this->_itemGrupo, $this->_listaNrCalculoFolha);
        $this->_caged->setAmbienteSalarial($this->_ambienteSalarial);
        $this->_caged->setTpModalidadeSalarioOperador($this->_tpModalidadeSalarioOperador);

    }


    public function _inicializaVariaveis($nrFormulario) {

        $this->_buscaDadosFormulario($nrFormulario);
        $this->_itemGrupo->carregaItensGrupoPorGrupoIncidencia($this->_nrOrg, $this->_nrOrgPadrao, $this->_nrGrpIncide);
        $this->_listaNrCalculoFolha = $this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
            $this->_nrOrg, $this->_tipoMovimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
        );
        $this->_msgSucessoPersisteMagnetico = null;
        $this->_msgSucessoGeraMagnetico = null;
        // Inicia o nome do arquivo com o default.
        $this->_nomeArquivo = 'caged.txt';
    }

    /**
     * Percorre os vinculos para geracao dos registros.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     */
    public function percorreCompetenciaMensal(){
        $this->_geraCAGED();
    }

    public function percorreRetrocesso(){
        $this->_caged->retroceder();
    }

    /* Gerar os registros do arquivo CAGED.
     * @throws Exception Propaga Excecao.
     * @return void
     * @author thalesvalias - 22/05/2012
     */

    private function _geraCAGED(){
//        percorre os vinculos gerando os registro do CAGED.
        if ($this->_idProcessaVinculo === true) {
//            inicializa Lista do Magnetico.
            $this->_cagedMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroCAGED::LISTA_TIPO_REG_EMP, $this->_nrOrg
            );

//            @todo deve retornar apenas os vinculos demitidos ou admitidos de acordo com o mes de competencia?
//            Retorna todos os vinculos.
            $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
                $this->_nrOrg, $this->_mesCompetencia, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, false,
                $this->_nrEstruturaTrab
            );

            if (empty($vinculos) !== true) {
//                percorre os vinculos gerando caged.
                foreach ($vinculos as $vinculo) {
                    if ($vinculo->getNrvinculoempreg() !== false) {
                        $this->_caged->geraCAGED($vinculo, $this->_tpInscHierarq);
                    }else{
                        throw new Exception('Vinculo empregaticio nao encontrado para o vinculo '.$vinculo->getNrvinculom());
                    }
                }
            }else{
                throw new Exception('Não foram encontrados vínculos.');
            }

            // Modifica o total de vinculos existentes na estrutura no inicio do primeiro dia do mes de competencia.
            $this->_caged->modificaTotalVinculos();

            // So persiste se a lista nao estiver vazia.
            $qtdRegistros = count($this->_cagedMagnetico->getItems());
            if ($qtdRegistros > 0) {
                $persisteCaged = $this->_cagedMagnetico->persist();
                if ($persisteCaged === true) {
                    $this->_msgSucessoPersisteMagnetico = "CAGED gravado com sucesso!";
                }
            }
        }

        // Gera o magnetico.
        if ($this->_idGeraMeioMagnetico === true) {
            $this->_geraMagnetico();
        }
    }


    /**
     * Enter description here ...
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     */
    private function _geraMagnetico(){

        // Buscar da base os resgistros daquela competencia, ordenados por  inscricao  e tipo de registro.
        $resultado = $this->_cagedMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg);

        if (count($resultado) > 0) {

            // Define o nome do arquivo.
            $this->_constroiNomeArquivo();

            // Monta o diretorio completo, incluindo o nome do arquivo a ser gerado.
            $caminhoArquivo = $this->_filePath."\\".$this->_nomeArquivo;

            // Criar o arquivo.
            //$arquivo = new TkFile($caminhoArquivo, true);

            //Chama método para gerar o arquivo, passando os parâmetros: nome da pasta, nome do arquivo e extensão arquivo
            $nomePasta = "CAGED";
            $nomeArquivo = "CAGED".$this->_mesCompetencia->toString(TkDateTime::YEAR).".Mm";
            $extensao = ".Mm";
            $arquivo = \FPA::geraArquivo($nomePasta, $nomeArquivo, $extensao);

            $this->_preparaRegistroA();

            /* Informacoes sobre o responsavel do arquivo. */
            $this->_registroA->geraRegistroA();

            // Armazenar o registro A com quebra de linha.
            $arquivo->add($this->_registroA->getLinhaRegistro().RegistroCAGED::QUEBRA_LINHA_CR_LF);

            $contador = 2;
            // Loop no resultado e armazenar os registros com quebra de linha.
            foreach ($resultado as $item) {
                // Metodo chamado dentro do loop, pois o resultado dos registros ja vem ordenado.
                $linha = $this->_caged->modificaSequencia($item->getDslinharegistro(), $item->getIdtiporegistro(), $contador);
                $item->setDslinharegistro($linha);
                $item->persist();

                $arquivo->add($linha);

                // Nao adiciona quebra de linha apos inclusao da ultima linha.
                if (end($resultado) !== $item) {
                    $arquivo->add(RegistroCAGED::QUEBRA_LINHA_CR_LF);
                }

                $contador++;
            }

            $arquivo->save(false, null, 0777);

            $arquivo->download(substr($_SERVER['REQUEST_URI'], 0, strlen($_SERVER['REQUEST_URI']) - 12)."/".\FPA::getNrorgTrabalho()."/".$nomePasta."/".$nomeArquivo);

//            if (empty($geraArquivo) === false) {
//                $this->_msgSucessoGeraMagnetico =  "Arquivo gerado com sucesso!";
//            }
        } else {
            tk_warning("Vínculo ".$this->_vinculom->getNrvinculom()." não possui registros.");
        }
    }


    /**
     * Constroi o nome do arquivo magnetico do caged.
     *
     * @return void
     *
     * @author thalesvalias - 01/06/2012
     */
    private function _constroiNomeArquivo() {
        $anoCompetencia = $this->_mesCompetencia->toString(TkDateTime::YEAR);
        $mesCompetencia = $this->_mesCompetencia->toString(TkDateTime::MONTH);
        // Retira o 0 do inicio do numero, transformando-o em inteiro.
        $mesCompetencia = intval($mesCompetencia);

        // Verifica se o arquivo a ser gerado sera de acerto ou nao. O tratamento do nome varia de acordo com o tipo de atualizacao.
        if ($this->_idAtualizacao === FpaFormcagedEntityBase::IDATUALIZACAO_NORMAL) {
            $this->_nomeArquivo = self::PREFIXO_NOME_ARQUIVO_NORMAL.$anoCompetencia.self::PREFIXO_EXTENSAO_ARQUIVO.$mesCompetencia;
        } else {
            $this->_nomeArquivo = self::PREFIXO_NOME_ARQUIVO_ACERTO.$anoCompetencia.self::PREFIXO_EXTENSAO_ARQUIVO.$mesCompetencia;
        }
    }

    private function _preparaRegistroA(){

        if (empty($this->_nrEstruturaResp) !== true) {
            $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);
            $estrutura->endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);

            if ($this->_tpInscResp === RegistroCAGED::CNPJ) {
                $this->_registroA->setTpInscricao(RegistroCAGED::CNPJ);
                $this->_registroA->setNrInscricao($estrutura->historico->getCdcnpjestrut());
            } elseif ($this->_tpInscResp === RegistroCAGED::CEI) {
                $this->_registroA->setTpInscricao(RegistroCAGED::CEI);
                $this->_registroA->setNrInscricao($estrutura->historico->getCdceiestrut());
            }

            $totalRegistroB = $this->_cagedMagnetico->totalRegistrosPorTipo($this->_mesCompetencia, $this->_nrOrg, RegistroBCAGED::REGISTRO_B);
            $totalRegistroC = $this->_cagedMagnetico->totalRegistrosPorTipo($this->_mesCompetencia, $this->_nrOrg, RegistroCCAGED::REGISTRO_C);
            $totalRegistroX = $this->_cagedMagnetico->totalRegistrosPorTipo($this->_mesCompetencia, $this->_nrOrg, RegistroXCAGED::REGISTRO_X);

            $totalMovimentacoes = $totalRegistroC + $totalRegistroX;

            $this->_registroA->setEstrutura($estrutura);
            $this->_registroA->setCompetencia($this->_mesCompetencia);
            $this->_registroA->setTotalEstabelecimento($totalRegistroB);
            $this->_registroA->setTotalMovimentacao($totalMovimentacoes);
            $this->_registroA->setAlteracao(self::ALTERACAO);
        } else {
            tk_warning('Estrutura responsável não selecionada.');
        }
    }

    /**
     * Busca dados do formulario.
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @return void
     *
     * @author thalesvalias - 22/05/2012
     */
    private function _buscaDadosFormulario($nrFormulario){
        $formulario = new FpaFormcagedEntity();
        $this->_formulario = $formulario->retornaDadosFormularioCAGED($this->_nrOrg, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_dsInicio = $this->_formulario->getDsinicio();
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_nrTpHierarquiaOrg = $this->_formulario->getNrhierarqorg();
            $this->_tpInscHierarq = $this->_formulario->getIdtpinschierarq();
            $this->_idProcessaVinculo = $this->_formulario->getIdprocvinculo();
            $this->_idGeraMeioMagnetico = $this->_formulario->getIdgerameiomag();
            $this->_filePath = $this->_formulario->getDscaminhoarquivo();
            $this->_nrEstruturaResp = $this->_formulario->getNrestrutrespons();
            $this->_tpInscResp = $this->_formulario->getIdtpinscrespons();
            $this->_nrGrpIncide = $this->_formulario->getNrgrpincide();
            $this->_idAtualizacao = $this->_formulario->getIdatualizacao();
        }
    }


}